MazeAnimation = function(pixelScale, maze, robot) {
  this.pixelScale = pixelScale;
  this.maze = JSON.parse(JSON.stringify(maze));

  this.actionQueue = [];


  this.deg = robot.deg;
  this.x = robot.x;
  this.y = robot.y;

  this.drawRotatedImage = function(p, img, px, py, theta) {
    // loadImage is async, so test if the image has been loaded
    // before drawing
    //console.log("drawRotated");
    if(img.width > 0) {
      //console.log("drawing robot");

      p.translate(px, py);
      p.rotate(-theta * p.TWO_PI / 360);
      p.translate(-img.width/2, -img.height/2);
      p.image(img, 0, 0);

    }
  };


  this.draw = function(p) {
    // draw the maze
    p.background(220, 240, 255);
    for(i = 0; i < maze.wallList.length; i++) {
      var x = maze.wallList[i].x;
      var y = maze.wallList[i].y;
      var px = x * this.pixelScale + 2;
      var py = (maze.height - y) * this.pixelScale - 2;

      p.rect(px, py, this.pixelScale - 4, -(this.pixelScale - 4));
    }

    // draw the robot

    if(this.robotImg.width > 0) {
      var px = (this.x + .5) * this.pixelScale;
      var py = (this.maze.height - this.y - .5) * this.pixelScale;
      //console.log(px);
      //console.log(py);
      this.drawRotatedImage(p, this.robotImg, px, py, this.deg);

      if(this.actionQueue.length > 0) {
        this.actionQueue.shift()();
      }
    }
  };

  // motion animations
  this.forward = function() {
    var t2 = function() {
      rad = this.deg * Math.PI * 2 / 360;
      //console.log(this);
      this.x += .02 * Math.cos(rad);
      this.y += .02 * Math.sin(rad);
    };
    for(var i = 0; i < 50; i++) {
        this.actionQueue.push(t2.bind(this));
    }
  };

  this.left = function() {
    var r2 = function() {
      this.deg += 2;
      //console.log("inc deg " + this.deg);
    };


    for(var i = 0; i < 45; i++) {
      //console.log("pushed");
      this.actionQueue.push(r2.bind(this));
    }
  };

  this.right = function() {
    var rn2 = function() {this.deg -= 2;};

    for(var i = 0; i < 45; i++) {
      this.actionQueue.push(rn2.bind(this));
    }
  };

};


// width and height in robot units
Maze = function(width, height) {
  this.width = width;
  this.height = height;

  this.wallList = [];

  this.addWall = function(x, y) {
    this.wallList.push({x: x, y: y});
  };

  this.blocked = function(x, y) {
    // position out of bounds
    if( x < 0 || y < 0 || x >= this.width || y >= this.height) return true;

    // check if position contains a wall
    for(i = 0; i < this.wallList.length; i++) {
      if(this.wallList[i].x === x && this.wallList[i].y === y) {
        return true;
      }
    }
    return false;

  }

};


Robot = function(x, y, deg, maze) {
  this.x = x;
  this.y = y;
  this.deg = deg;

  this.maze = maze;

  this.left = function() {
    this.deg = (this.deg + 90) % 360;
  };

  this.right = function() {
    this.deg = (this.deg + 270) % 360;
  }

  this.forward = function() {
    var rad = this.deg * Math.PI * 2 / 360;
    //console.log(this);
    this.x += Math.cos(rad);
    this.y += Math.sin(rad);

    this.x = Math.round(this.x);
    this.y = Math.round(this.y);
  }


  this.blocked = function() {
    var rad = this.deg * Math.PI * 2 / 360;
    var nextX = Math.round(this.x + Math.cos(rad));
    var nextY = Math.round(this.y + Math.sin(rad));

    //console.log("nextX " + nextX);
    //console.log("nextY " + nextY);
    return maze.blocked(nextX, nextY);

  }
}


// stub to have processing redraw maze and robot every frame
// relies on global variable mazeAnimation

var sketchProc=function(p){


  mazeAnimation.robotImg = p.loadImage("drone", "png");
  p.size(640, 640);

  left = function() {
    mazeAnimation.left();
    robot.left();
  }

  right = function() {
    mazeAnimation.right();
    robot.right();
  }

  forward = function() {
    mazeAnimation.forward();
    robot.forward();
  }

  blocked = function() {
    return robot.blocked();
  }
  /*
  left();
  forward();
  right();
  forward();
*/
  //robot.initGraphics(p);

  p.draw = function() {
    mazeAnimation.draw(p);
  };

}; // end processing code
